//
// C++ Implementation: matrix_eigen_test
//
// Description: 
//
//
// Author: David Saxton, Alan Grimes, Zoltan Padrah <zoltan.padrah@gmail.com>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

// temporarly:
#define USE_EIGEN_LU_IMPLEMENTATION

#include "matrix_2.h"

using namespace std;

double gen_random();

int main(void){
    
    const int SIZE = 5;
    
    // init rand. number generator: 
    srand( time(NULL) );
    
    Matrix m(SIZE);
    
    // fill it with random data
    int i,j;
    for(i=0; i<SIZE; i++)
        for(j=0; j<SIZE; j++)
            m.g(i,j) = rand()%100;
    
    // print it
    cout << "m:\n";
    m.displayMatrix(cout);
    cout << "\n";
    
    // call decomposition
    m.performLU();
    
    cout << "m decomposed:\n";
    m.displayLU(cout);
    cout << "\n";
    
    // some elements here
    QuickVector b(SIZE);
    QuickVector x(SIZE);
    
    // fill it
    for(i=0; i<SIZE; i++)
        b(i) = i;
    x = b;
    
    cout << "b = x = \n" << b << "\n";
    
    // solve
    m.fbSub(&x);
    
    cout << "x, result :\n" << x << "\n";
    
    // check
    QuickVector c(SIZE);
    m.multiply(&x, &c);
    
    cout << "A*x = \n" << c << "\n";
    
    // error ?
    double error = 0;
    for(i=0; i<SIZE; i++)
        error += abs( c(i) - b(i) );
    
    cout << "Error: " << error << endl ;
}
